
from numpy.core.numerictypes import *
import numpy
from pgplot import *
from sets import *
from NumRec import *
from masks import *
from scipy.special import *
from nnls import *
from basics import *

DATABASE = os.environ["PYTHONDATA"]+"/photometry"
SITEGAINS = [0.82,0.85,0.87,1.13,0.77,0.93,0.92,0.80]
E2VGAINS = 8*[0.8]

LDPDISP = { "lambda": 100.0*array([38,39,40,41,42,43,44,45,46,47,48,
            49,50,51,52,53,54,55,56,57,58,59,60, 62,64,66,68,70,72,74,76,78,80,82,84,
            86,88,90,92,94,96,98,100,102,104,106]),
            "position": array([2269,2033,1823,1635,1467,1315,1177,1052,
            938,834,738,650,569,494,424,360,300,243, 191,142,96,53,12,-61,-127,-186,-238,-285,
            -327,-365,-400,-431,-459,-484,-507,-528, -547,-564,-579,-593,-606,-617,-628,-637, -645,-652])}

LDPCOEF = array([ 0.48682052, 0.26810576,-0.19797286,-0.02573573,-0.02867552,-0.03297195,
      -0.00763004,-0.00584553,])

LDPZ = 75.0

UDPDISP = { "lambda": 100.0*array([40,45,50,55,60,65,70,75,80,85,90,95,100,105,110],float),
            "position": array([7679,7103,6711,6425,6206,6032,5886,5762,5653,5554,5463,5378,5296,5215,5137],float)}

UDPCOEF = array([0.0091252256358470701, 0.0086607268458589251, 0.00029583655313776894,
        0.00011883092384866499, -0.00013490781129557009, -0.00027215263970278456,
        -7.6633237193462675e-05, -5.1710797907711641e-05])
UDPORDER = 7
UDPZ = 65.0

if not os.environ.has_key("UDP"):
   UDP = False
else:
   UDP = True
   LDPCOEF = UDPCOEF
   LDPCOEF = UDPCOEF
   LDPZ = UDPZ

LDPCOEF0 = 1e4/3.8e3
LDPCOEF1 = 1e4/13e3
LDPCOEFC = (LDPCOEF0+LDPCOEF1)/2.
LDPCOEFS = (LDPCOEF0-LDPCOEF1)/2.
LDPCOEFCi = (13e3+3.8e3)/2.
LDPCOEFSi = (13e3-3.8e3)/2.

ROTepoch = 2007.37

sstr = lambda s: s.strip()

class SMFDict(DictType):
    SMFKEYS = ["type","id","ra","dec","x","y","w","t","b"]
    fn = [sstr,sstr,sstr,sstr,float,float,float,float,float]
    def ReadFile(self,smf_file):
        self.name = os.path.basename(smf_file).split(".")[0]
        for key in self.SMFKEYS: self[key] = []
        f = open(smf_file,"r")
        for l in f:
           data = l.split()
           if data[0] == "POSITION":
              self["center"] = data[2:4]
              self["epoch"] = float(data[4])
              self["angle"] = float(data[5])
           elif (data[0] == "SLIT") or (data[0] == "HOLE" and data[7] == "1"):
              extra = lambda key: (key in "tb" and data[7] == "1" and 1) or 0
              [self[key].append(self.fn[self.SMFKEYS.index(key)](data[self.SMFKEYS.index(key)+extra(key)])) for key in self.SMFKEYS]
        f.close()
        for key in self.SMFKEYS[4:]: self[key] = array(self[key])
        self["spec"] = [None for n in range(len(self["id"]))]

def ReadChip(root,n,sex=0,sexdir="./",M=3,N=9,FW=1.0,direct=False):
   cat = "%sc%d.cat" % (root,n)
   if sex or not os.path.exists(cat):
     image = "%sc%d.fits" % (root,n)
     fsexdir=sexdir
     if fsexdir[-1] != "/": fsexdir = sexdir + "/"
     pid = os.getppid()
     filtname = "simple%d.conv" % (pid)
     kernelM = exp(-0.5*(arange(-M,M+0.1,1,float32))**2/(FW/2.35)**2)
     kernelN = ones(arange(-N,N+0.1,1,float32).shape,float64)
     kernel = kernelN[::,NewAxis]*kernelM[NewAxis,::]
     filtout = open(filtname,"w")
     print >> filtout, "CONV NORM"
     print >> filtout, "# %dx%d simple convolution mask" % (len(kernelM),len(kernelN))
     fmt = len(kernelM)*" %.7f"
     for k1 in kernel: print >> filtout, fmt % tuple(k1)
     filtout.close()
     tval = 100
     F = FITS(image)
     if F["ccd-type"] == "E2V": tval = 20
     F.close()
     command = " ".join(["sex",image,"-c",fsexdir+"default.sex",
            "-CATALOG_NAME",cat,
            "-DETECT_THRESH %f" % ([5,tval][direct]),
            "-DETECT_MINAREA %d" % ([N,N][direct]),
            "-ANALYSIS_THRESH %f" % ([2,20][direct]),
            "-DEBLEND_NTHRESH %d" % ([64,64][direct]),
            "-GAIN 2",
            "-BACK_SIZE %d" % ([5,64][direct]),
#            "-BACKFILTER_SIZE %d" % ([5,3][direct]),
            "-PIXEL_SCALE 0.2",
            "-SEEING_FWHM 1.0",
            "-FILTER_NAME",filtname,
            "-STARNNW_NAME",fsexdir+"default.nnw",
            "-PARAMETERS_NAME",fsexdir+"default.param"
            ])
     verbex(command)
     os.remove(filtname)
   cx,cy,cm = transpose([map(float,l.split()[1:4]) for l in open(cat,"r").readlines() if "#" not in l]) - 1
   ca = exp10(-0.4*cm)
   return (cx,cy,ca)

def OneGaussian(p,x):
   c,s = p
   g = exp(-0.5*((x-c)/s)**2)
   return g

def MultiGaussian(p,c0,s0,x,lam,dlam,y,n,w,fit=1,verb=0):
   l = len(c0)
   c = p[:l]
   c = where(equal(w,0),c0,c)
   cpars = p[l:]
   cont = (cpars[0]+cpars[1]*dlam) * exp(-(lam-cpars[2])*greater(lam,cpars[2])/cpars[3])
   s = power(10,s0)
   m = len(c)
   p1 = lambda j: [c[j],s[j]]
   b = asarray([OneGaussian(p1(j),x) for j in range(m)] + [cont])
   sol = nnls(b*n[NewAxis,::],y*n)[1]
#   print sol
   a = sol[:m]
   g = dot(sol,b)
   if verb:
      if verb>1:
        for j in range(m): print j+1,a[j],p1(j)
      else:
          print abs(verb)+1,a[abs(verb)],p1(abs(verb))
#   exit()
#   g = sum([OneGaussian(p1(j),x) for j in range(m)]) + cont
   if fit:
      d = g-y
      h = d*n
      return h
   else:
      return g,cont,a

def AdjustSpacings(A,s=1e-6,extras=None,verb=1):
   x = A[0]
   o = argsort(x)
   A = take(A,o,1)
   if extras: extras = [take(extra,o) for extra in extras]
   x = A[0]
   dx = diff(x)
   while sometrue(equal(dx,0)):
      if verb: print "adjusting spacings...", sum(equal(dx,0))
      p = concatenate([[0.0],equal(dx,0)])
      x = x + s * p * (uniform(0,1,p.shape)-0.5)
      A[0] = x
      o = argsort(x)
      A = take(A,o,1)
      if extras: extras = [take(extra,o) for extra in extras]
      x = A[0]
      dx = diff(x)
   if extras: return A,extras
   else: return A


def ghquad(w,s,tck,X,W):
   s2 = sqrt(2)*s
   x = s2 * X
   f = splev(x+w,tck)
   k2 = pow(x/s2,2)
   h = exp(-k2) * W
   i = sum(h*f)
   i = i/sqrt(pi)
   return i

def ResolutionPlots():
   opendev(PGPLOT_DEV,nx=2,ny=2,land=0,size=6,asp=1)
   cpgscr(0,1,1,1)
   cpgscr(1,0,0,0)
   x = UDPDISP["position"]/15.
   y = UDPDISP["lambda"]
   x = x - x[searchsorted(y,6000)]
   x0 = LDPDISP["position"]/15.
   y0 = LDPDISP["lambda"]
   x0 = x0 - x0[searchsorted(y0,6000)]

   newbox([-80,200],[3800,11e3],ch=2)
   xlabel("x (pixels)")
   ylabel("\gl (\A)")
   drawpt(x,y,ps=4,ch=2)
   drawpt(x0,y0,ps=17,ch=2)

   ya = arange(3800,12.0e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,len(LDPCOEF)-1)
   xa = LDPZ*log(dot(LDPCOEF,lb))+LDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   drawln(xa,ya,ls=2)

   lx = (UDPDISP["lambda"] - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,UDPORDER)
   wt = 1/(UDPDISP["lambda"]/1e3)
   udpsol = svdfit(transpose(lb)*wt[::,NewAxis],wt*exp(-(UDPDISP["position"]/15-UDPZ)/UDPZ))
   print udpsol.tolist()
#   udpsol = UDPCOEF
   ya = arange(3800,12e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,UDPORDER)
   xa = UDPZ*log(dot(udpsol,lb))+UDPZ
   xa = -xa
   xa = xa - xa[searchsorted(ya,6000)]
   drawln(xa,ya,ls=1)

   newbox([-80,200],[-2,2],ch=2)
   xlabel("x (pixels)")
   ylabel("\gDx (pixels)")

   ya = y0
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,len(LDPCOEF)-1)
   xa = LDPZ*log(dot(LDPCOEF,lb))+LDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   dx0 = x0 - xa
   drawpt(xa,dx0,ps=17,ch=2)

   ya = y
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,UDPORDER)
   xa = UDPZ*log(dot(udpsol,lb))+UDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   dx = x - xa
   drawpt(xa,dx,ps=4,ch=2)

   newbox([3800,11e3],[0,500],ch=2)
   xlabel("\gl (\A)")
   ylabel("d\gl/dx (\A/pixel)")

   ya = arange(3800,11.01e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,len(LDPCOEF)-1)
   xa = LDPZ*log(dot(LDPCOEF,lb))+LDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   tck0 = splrep(xa[::-1],ya[::-1],k=1,s=0)
   dldx0 = -splev(xa[::-1],tck0,1)[::-1]
   drawln(ya,dldx0,ls=2)

   ya = arange(3800,11.01e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,UDPORDER)
   xa = UDPZ*log(dot(udpsol,lb))+UDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   tck = splrep(xa[::-1],ya[::-1],k=1,s=0)
   dldx = -splev(xa[::-1],tck,1)[::-1]
   drawln(ya,dldx,ls=1)

   fw = 4.5
   newbox([3800,11e3],[0,100],ch=2)
   xlabel("\gl (\A)",ch=2)
   ylabel("R (%.1f pixels)" % (fw),ch=2)

   ya = arange(3800,11.01e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,len(LDPCOEF)-1)
   xa = LDPZ*log(dot(LDPCOEF,lb))+LDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   tck0 = splrep(xa[::-1],ya[::-1],k=1,s=0)
   dldx0 = -splev(xa[::-1],tck0,1)[::-1]
   r0 = (ya/(fw*dldx0))
   drawln(ya,r0,ls=2)

   ya = arange(3800,11.01e3,100,float)
   lx = (ya - LDPCOEFCi)/LDPCOEFSi
   lb = legendre(lx,UDPORDER)
   xa = UDPZ*log(dot(udpsol,lb))+UDPZ
   xa = xa - xa[searchsorted(ya,6000)]
   xa = -xa
   tck = splrep(xa[::-1],ya[::-1],k=1,s=0)
   dldx = -splev(xa[::-1],tck,1)[::-1]
   r = (ya/(fw*dldx))
   drawln(ya,r,ls=1)

   closedev(pstopdf=True)

if __name__ == "__main__":
   ResolutionPlots()



