
from FITS import FITS
from SimpleQueue import *
from basics import *

from twisted.internet import abstract, threads, task
from twisted.python import log as logging
from twisted.application import service, internet
from twisted.python.components import registerAdapter

from nevow               import inevow
from nevow               import accessors
from nevow               import appserver
from nevow               import athena
from nevow               import rend
from nevow               import loaders
from nevow               import url
from nevow               import tags as T
from nevow               import flat
from nevow               import util
from nevow               import livepage
from nevow               import canvas
from nevow               import static

from twisted.internet import task
from twisted.internet import defer, protocol, reactor, address, utils

import time, sys, os
from glob import glob

from twisted.python.util import sibpath
from twisted.python.filepath import FilePath

sibling = FilePath(__file__).sibling

class RootPage(athena.LivePage):
    def __init__ ( self, *args, **kwargs ):
        self.runner = kwargs["runner"]
        kwargs.pop("runner")
        athena.LivePage.__init__ ( self, *args, **kwargs )
        self.docFactory = self.makeDocFactory()
        self.jsModules.mapping[u"EchoThing"] = sibling("HubSite.js").path

    def locateChild(self,ctx, segments):
        request = inevow.IRequest(ctx)
        if len(segments)>1 and os.path.exists(self.runner.basedir+"/".join(segments)):
           I = None
           suff = segments[-1].split(".")[-1]
           if suff in ["jpg","png","gif"]:
              I = static.File( self.runner.basedir+"/".join(segments),"image/%s" % (suff.replace("jpg","jpeg")))
           elif suff in ["txt","dat"]:
              I = static.File( self.runner.basedir+"/".join(segments),"text/plain")
           elif suff == "fits":
              I = static.File( self.runner.basedir+"/".join(segments),"application/fits")
           if I:
              print I.fp
              return I,()
           else:
              return athena.LivePage.locateChild(self,ctx,segments)
        return athena.LivePage.locateChild(self,ctx,segments)

    def problem(self,reason):
        logging.err(reason)
        reason.printTraceback(sys.stdout)

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        self.targetsTable.disconn(reason)
        self.targetWatcher.disconn(reason)
        self.pipelineStatus.disconn(reason)

    def render_ticker(self,ctx,data):
        obj = FrameTicker(runner=self.runner)
        obj.setFragmentParent(self)
        self.frameTicker = obj
        return ctx.tag[obj]

    def render_status(self,ctx,data):
        obj = PipelineStatus(runner=self.runner)
        obj.setFragmentParent(self)
        self.pipelineStatus = obj
        return ctx.tag[obj]

    def render_targets(self,ctx,data):
        obj = TargetsTable(runner=self.runner)
        obj.setFragmentParent(self)
        self.targetsTable = obj
        return ctx.tag[obj]

    def render_watcher(self,ctx,data):
        obj = TargetWatcher(runner=self.runner)
        obj.setFragmentParent(self)
        self.targetWatcher=obj
        return ctx.tag[obj]

    def makeDocFactory ( self ):
        return loaders.stan(T.html[
                T.head(render=T.directive('liveglue'))[
                  T.title["NEWFIRM Runner Main Page"]],
                  T.body[
                     T.table[
                     T.tr[
                       T.td[
                         T.table[
                         T.tbody[
                           T.tr[
                            T.td(style=u"height:160;border:thin solid black;")[T.div(render=T.directive("ticker"))],
                            ],
                           T.tr[
                            T.td(style=u"height:30;border:thin solid black;")[T.div(render=T.directive("status"))],
                            ],
                           T.tr[
                            T.td(style=u"border:thin solid black;")[T.div(render=T.directive("targets"))],
                            ]
                         ]
                         ]
                         ],
                       T.td(style=u"height:640;border:thin solid black;")[T.div(render=T.directive("watcher"))],
                     ],
                     ],
                     ]
                ])

class MainPage(RootPage):
    def child_( self, ctx):
        return RootPage(runner=self.runner)

class FrameTicker(athena.LiveElement):

    jsClass = u'EchoThing.EchoWidget'

    def __init__(self,*args,**kwargs):
        self.runner = kwargs["runner"]
        kwargs.pop("runner")
        athena.LiveElement.__init__ ( self, *args, **kwargs )
        self.docFactory = self.makeDocFactory()
        self.ifiles = glob(self.runner.datadir+"/image?????.fits")
        self.ifiles.sort()

    def problem(self,reason):
        logging.err(reason)
        reason.printTraceback(sys.stdout)

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        if hasattr(self,"clock") and self.clock.running: self.clock.stop()

    def fireItUp( self):
        self.clock = task.LoopingCall(self.loop)
        self.clock.start(1)
        self.i = 0
        self.last = None

    def loop( self):
        if hasattr(self.page,"callRemote"):
           ifiles = [g for g in glob(self.runner.datadir+"/image?????.fits") if g not in self.ifiles]
           if not ifiles:
              ifiles = self.ifiles
           ifiles.sort()
           if ifiles:
              ifile = ifiles[-1]
              if ifile != self.last:
                 F = FITS(ifile)
                 ra = F["ra"]
                 dec = F["dec"]
                 exptime = F["exptime"]
                 F.close()
                 js = []
                 for c in [1,2,3,4]:
                     b = os.path.basename(ifile).replace(".fits","c%d.jpg" % (c))
                     js += ["jpegs/"+b]
                 go=lambda c: self.runner.TickOne(ifile,c)
                 map(go,[1,2,3,4])
#                 RunForkedQueue(go,[1,2,3,4],MaxThreads=4,monitor=0,verb=0)
                 j1,j2,j3,j4 = js
#                 j1 = "jpegs/"+os.path.basename(self.runner.TickOne(ifile,1))
#                 j2 = "jpegs/"+os.path.basename(self.runner.TickOne(ifile,2))
#                 j3 = "jpegs/"+os.path.basename(self.runner.TickOne(ifile,3))
#                 j4 = "jpegs/"+os.path.basename(self.runner.TickOne(ifile,4))
                 b = os.path.basename(ifile)
                 if self.i:

                    self.callRemote('delText', u'ticker1', u'name')
                    self.callRemote('delText', u'ticker1', u'ra')
                    self.callRemote('delText', u'ticker1', u'dec')
                    self.callRemote('delText', u'ticker1', u'exptime')

                    self.callRemote('delText', u'chip1', u'chip1im')
                    self.callRemote('delText', u'chip2', u'chip2im')
                    self.callRemote('delText', u'chip3', u'chip3im')
                    self.callRemote('delText', u'chip4', u'chip4im')

                    self.callRemote('delText', u'ticker2', u'chip1')
                    self.callRemote('delText', u'ticker2', u'chip2')
                    self.callRemote('delText', u'ticker2', u'chip3')
                    self.callRemote('delText', u'ticker2', u'chip4')

                 self.callRemote('addElement', u'td', u'', u'ticker1', u'%s' % (b), u'name')
                 self.callRemote('addElement', u'td', u'', u'ticker1', u'%s' % (ra), u'ra')
                 self.callRemote('addElement', u'td', u'', u'ticker1', u'%s' % (dec), u'dec')
                 self.callRemote('addElement', u'td', u'', u'ticker1', u'%.1f' % (exptime), u'exptime')

                 self.callRemote('addElement', u'td', u'', u'ticker2', u'', u'chip1')
                 self.callRemote('addElement', u'td', u'', u'ticker2', u'', u'chip2')
                 self.callRemote('addElement', u'td', u'', u'ticker2', u'', u'chip3')
                 self.callRemote('addElement', u'td', u'', u'ticker2', u'', u'chip4')
                 self.callRemote('addImage', u'chip1', u'%s'%(j1), u'chip1im')
                 self.callRemote('addImage', u'chip2', u'%s'%(j2), u'chip2im')
                 self.callRemote('addImage', u'chip3', u'%s'%(j3), u'chip3im')
                 self.callRemote('addImage', u'chip4', u'%s'%(j4), u'chip4im')
                 self.callRemote('changeElement', u'style', u'width:90', u'chip1im')
                 self.callRemote('changeElement', u'style', u'width:90', u'chip2im')
                 self.callRemote('changeElement', u'style', u'width:90', u'chip3im')
                 self.callRemote('changeElement', u'style', u'width:90', u'chip4im')

                 self.i += 1
                 self.last = ifile

    def makeDocFactory( self):
        self.fireItUp()
        return loaders.stan(T.div(render=T.directive('liveElement'))[
             T.h2["Frame Ticker"],
             T.table(style="border:thin solid black;")[
             T.thead[ T.tr[
                T.th(style="border:thin solid black;width:90")["Frame"],
                T.th(style="border:thin solid black;width:90")["RA"],
                T.th(style="border:thin solid black;width:90")["DEC"],
                T.th(style="border:thin solid black;width:30")["EXPTIME"],
                ] ],
             T.tbody[
                T.tr(name="ticker1")
                ],
                ],
             T.table(style="border:thin solid black;")[
             T.thead[ T.tr[
                T.th(style="border:thin solid black;width:75")["1"],
                T.th(style="border:thin solid black;width:75")["2"],
                T.th(style="border:thin solid black;width:75")["3"],
                T.th(style="border:thin solid black;width:75")["4"],
                ] ],
             T.tbody[
                T.tr(name="ticker2")
                ],
                ],
                ])


class PipelineStatus(athena.LiveElement):

    jsClass = u'EchoThing.EchoWidget'

    def __init__(self,*args,**kwargs):
        self.runner = kwargs["runner"]
        kwargs.pop("runner")
        athena.LiveElement.__init__ ( self, *args, **kwargs )
        self.docFactory = self.makeDocFactory()

    def problem(self,reason):
        logging.err(reason)
        reason.printTraceback(sys.stdout)

    def check( self):
        self.clock = task.LoopingCall(self.loop)
        self.clock.start(1)

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        if hasattr(self,"clock") and self.clock.running: self.clock.stop()

    def loop( self):
        if hasattr(self.page,"callRemote") and self.runner.going != self.status:
           if self.runner.going:
              status = "Running"
              color = "green"
           else:
              status = "Paused"
              color = "red"
           d = self.callRemote('changeElement', u'style', u"color:%s"%(color), u"pstatus")
           d.addCallback(lambda x: self.callRemote('delText', u"pstatus", u"status"))
           d.addCallback(lambda x: self.callRemote('addText', u"span", u"pstatus", u"%s"%(status), u"status"))
        self.status = self.runner.going


    def select( self, value=None):
        self.runner.going = not self.runner.going
        self.loop()
    athena.expose(select)

    def makeDocFactory( self):
        self.check()
        self.status = self.runner.going
        click = u'Nevow.Athena.Widget.get(this).selected(\'switch\')'
        if self.runner.going:
           status = "Running"
           color = "green"
        else:
           status = "Paused"
           color = "red"
        return loaders.stan(T.div(render=T.directive('liveElement'))[
             T.h2[
             T.span["Pipeline Status:"],
             T.span(style=u"color:%s"%(color),name=u"pstatus",onclick=click)[
                T.span(name=u"status")[status]
                ]
                ],
                ])


class TargetsTable(athena.LiveElement):

    jsClass = u'EchoThing.EchoWidget'

    def __init__(self,*args,**kwargs):
        self.runner = kwargs["runner"]
        kwargs.pop("runner")
        athena.LiveElement.__init__ ( self, *args, **kwargs )
        self.docFactory = self.makeDocFactory()
        self.qstats = {}

    def problem(self,reason):
        logging.err(reason)
        reason.printTraceback(sys.stdout)

    def check( self):
        self.current = None
        self.showing = []
        self.clock = task.LoopingCall(self.loop)
        self.clock.start(1)

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        if hasattr(self,"clock") and self.clock.running: self.clock.stop()

    def loop( self):
        if hasattr(self.page,"callRemote"):
           for targ in self.runner.targs:
              if targ:
                 if targ not in self.targets:
                    itarg = self.runner.targs.index(targ)+1
                    name = u"targ(%d)" % (itarg)
                    self.targets.append(targ)
                    name1 = u"targ(%d)1" % (itarg)
                    name2 = u"targ(%d)2" % (itarg)
                    cname1 = u"ctarg(%d)1" % (itarg)
                    cname2 = u"ctarg(%d)2" % (itarg)
                    cstyle1 = u"width:300;font-weight:normal"
                    cstyle2 = u"width:20;font-weight:normal"
                    click1 = u'Nevow.Athena.Widget.get(this).selected(\'%s\')' % (targ)
                    self.callRemote('addElement', u'tr', u'', u'targetTable', u'', name)
                    self.callRemote('addElement', u'td', u'', name, u'', cname1+"d")
                    self.callRemote('addElement', u'td', u'', name, u'', cname2+"d")
                    self.callRemote('addElement', u'div', cstyle1, cname1+"d", u'', name1)
                    self.callRemote('addElement', u'div', cstyle2, cname2+"d", u'', name2)
                    self.callRemote('addElement', u'div', cstyle1, name1, u'%s' % (targ), cname1)
                    self.callRemote('addElement', u'div', cstyle2, name2, u'%d' % (len(self.runner.groups[targ])), cname2)
                    self.callRemote('changeElement', u'onclick', click1, name1)
                    done1 = self.runner.Data1[targ].donefirst
                    done2 = self.runner.Data2[targ].donesecond
                    ongoing1 = self.runner.Data1[targ].ongoing
                    ongoing2 = self.runner.Data2[targ].ongoing
                 else:
                    itarg = self.runner.targs.index(targ)+1
                    name = u"targ(%d)" % (itarg)
                    name1 = u"targ(%d)1" % (itarg)
                    name2 = u"targ(%d)2" % (itarg)
                    cname1 = u"ctarg(%d)1" % (itarg)
                    cname2 = u"ctarg(%d)2" % (itarg)
                    bname1 = u"btarg(%d)1" % (itarg)
                    bname2 = u"btarg(%d)2" % (itarg)
                    self.runner.Data1[targ].Already()
                    self.runner.Data2[targ].Already()
                    done1 = self.runner.Data1[targ].donefirst
                    done2 = self.runner.Data2[targ].donesecond
                    ongoing1 = self.runner.Data1[targ].ongoing
                    ongoing2 = self.runner.Data2[targ].ongoing
                    if targ not in self.qstats or self.qstats[targ] != (done1,done2,ongoing1,ongoing2):
                       self.qstats[targ] = (done1,done2,ongoing1,ongoing2)
                       bstyle1 = u"width:300;font-weight:normal"
                       bstyle2 = u"width:20;font-weight:normal"
                       if done1 and not done2:
                          bstyle1 += u";color:orange"
                          bstyle2 += u";color:orange"
                       elif done1 and done2:
                          bstyle1 += u";color:green"
                          bstyle2 += u";color:green"
                       else:
                          bstyle1 += u";color:red"
                          bstyle2 += u";color:red"
                       if ongoing1 or ongoing2:
                          bstyle1 += u";font-weight:bold"
                       else:
                          bstyle1 += u";font-weight:normal"
                       content1 = u"%s" % (targ)
                       content2 = u"%d" % (len(self.runner.groups[targ]))
                       self.callRemote('changeElement', u'style', bstyle1, cname1)
                       self.callRemote('changeElement', u'style', bstyle2, cname2)

    def items( self,targ):
        groups = self.runner.groups
        running = self.runner.targ
        done1 = self.runner.Data1[targ].donefirst
        done2 = self.runner.Data2[targ].donesecond
        itarg = self.runner.targs.index(targ)+1
        name = u"targ(%d)" % (itarg)
        name1 = u"targ(%d)1" % (itarg)
        name2 = u"targ(%d)2" % (itarg)
        cname1 = u"ctarg(%d)1" % (itarg)
        cname2 = u"ctarg(%d)2" % (itarg)
        cstyle1 = "width:300;font-weight:normal;color:blue"
        cstyle2 = "width:20;font-weight:normal;color:blue"
        content1 = T.div(style=cstyle1,name=cname1)[u"%s" % (targ)]
        content2 = T.div(style=cstyle2,name=cname2)[u"%d" % (len(self.runner.groups[targ]))]
        item1 = T.td[T.div(onclick='Nevow.Athena.Widget.get(this).selected(\'%s\')' % (targ))[T.div(style="width:300",name=name1)[
                [content1]]
                ]]
        item2 = T.td[T.div(style="width:80",name=name2)[content2]]
        p = T.tr(name=name)[ item1, item2, ]
        self.targets.append(targ)
        return p

    def select( self, value=None):
        if value:
           otarg = self.page.targetWatcher.target
           if otarg in self.runner.targs:
              itarg = self.runner.targs.index(otarg)+1
              name1 = u"targ(%d)1" % (itarg)
              self.callRemote('changeElement',u"style",u"color:black;text-decoration:normal",name1)
           itarg = self.runner.targs.index(value)+1
           name1 = u"targ(%d)1" % (itarg)
           self.callRemote('changeElement',u"style",u"color:black;text-decoration:underline",name1)
           self.page.targetWatcher.fireUp(value)
    athena.expose(select)

    def makeDocFactory( self):
        self.loop()
        self.check()
        self.targets = []
        return loaders.stan(T.div(render=T.directive('liveElement'))[
             T.h2["Exposure Groups (Targets)"],
             T.table(style="border:thin solid black;",name="targetTable")[
             T.thead[ T.tr[ T.th(style="border:thin solid black;")["Grouping"], T.th(style="border:thin solid black;")["N"] ] ],
             T.tbody[
                (self.items(targ) for targ in self.runner.targs if targ),
                ],
                ]])

class TargetWatcher(athena.LiveElement):

    jsClass = u'EchoThing.EchoWidget'

    def __init__(self,*args,**kwargs):
        self.runner = kwargs["runner"]
        kwargs.pop("runner")
        self.target = "No Target Selected"
        self.showP = 0
        self.showS = 0
        self.schunk = 2
        self.npasses = [1,2]
        self.nstages = {1:[1,2,3,4,5],2:[1,2,3,4,5,6,7,8,9]}
        self.tstages = {1:["Calibrated Frames","Sky Subtracted Frames","Shift Plots",\
                                "Stacked Frames","Mosaicked Frames"],
                        2:["Calibrated Frames","Interpolated Frames","Wavelet Transforms",\
                                "Model Transforms","Model Sky Frames","Sky Subtracted Frames",\
                                        "Shift Plots","Stacked Frames","Mosaicked Frames"]}
        self.stale = True
        athena.LiveElement.__init__ ( self, *args, **kwargs )
        self.docFactory = self.makeDocFactory()
        self.images = {}

    def fireUp( self, target):
        if self.target != target:
           name = u"%s" % (self.target.replace(" ",""))
           if self.stale:
              self.callRemote('delText', u"selheader", u"seltarget")
              self.stale = False
           else:
              self.clearsnaps()
              self.callRemote('delText', u"selheader", u"seltarget"+name)
           self.showP = 0
           self.showS = 0
           style = u"color:orange"
           [self.callRemote('changeElement',u"style",style,u"sel%d"%(100*p+s)) for p in [1,2] for s in self.nstages[p]]
           self.callRemote('delText',u"hproducts",u"products")
           self.callRemote('addText',u'div',u"hproducts",u"Select a Stage",u"products")
        self.target = target
        name = u"%s" % (self.target.replace(" ",""))
#        self.callRemote('addText', u"div", u"hproducts", u"Products for %s" % (self.target), u"products"+name)
        self.callRemote('addText', u"div", u"selheader", u"Target: %s" % (self.target), u"seltarget"+name)

    def select( self, value=None):
        showP = int(value)/100
        showS = int(value)%100

        if self.target in self.runner.targs and value:
          name = u"%s" % (self.target.replace(" ",""))
          if hasattr(self,"callRemote"):
             style = u"color:green"
             item = u"sel%d"%(100*showP+showS)
             self.callRemote('changeElement',u"style",style,item)
#             self.callRemote('delText',u"hproducts",u"products")
#             text = u"Products for %s (Pass %d, %s)" % (name,showP,self.tstages[showP][showS-1])
#             self.callRemote('addText',u'div',u"hproducts",text,u"products")
             if self.showP and self.showS:
                style = u"color:orange"
                item = u"sel%d"%(100*self.showP+self.showS)
                self.callRemote('changeElement',u"style",style,item)
             self.showP = showP
             self.showS = showS
             self.addThumbs()
        else:
          self.showP = 0
          self.showS = 0
    athena.expose(select)

    def problem(self,reason):
        logging.err(reason)
        reason.printTraceback(sys.stdout)

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        pass
#        self.clock.stop()

    def fireUpProducts( self):
        if hasattr(self.page,"callRemote"):
#           self.clock = task.LoopingCall(self.loop)
#           self.clock.start(1)
           if self.images:
              self.clearsnaps()
           self.snapshot()

    def beforeRender( self, ctx):
        d = self.notifyOnDisconnect()
        d.addErrback(self.disconn)

    def disconn( self, reason):
        if hasattr(self,"clock") and self.clock.running: self.clock.stop()

    def clearsnap( self,j):
        pics = sorted(self.images.keys())
        for pic in pics[j*self.schunk:(j+1)*self.schunk]:
            lpic = pic.replace("."+pic.split(".")[-1],"link")
            finfo = self.images[pic]
            for k,ftext in enumerate(finfo):
                if ftext:
                   fwords = ftext.split()
                   nwords = len(fwords)
                   for l in range(nwords):
                       self.callRemote('delText', u'thumb%sinforow%d' % (pic,k+1), u'thumb%sinfo%d' % (pic,k*nwords+l))
            self.callRemote('removeRows',u"thumb%sinfotbl" % (pic))
            self.callRemote('delText',lpic,pic)
            self.callRemote('delText',u"thumb%s" %(pic),lpic)
            self.callRemote('delText',u"thumb%srow1" %(pic),u"thumb%s" % (pic))
            self.callRemote('delText',u"thumb%srow2" %(pic),u"thumb%sinfo" % (pic))
            self.callRemote('removeRows',u"thumb%stbl" % (pic))
            self.callRemote('delText',u"thumb%sitem" %(pic),u"thumb%stbl" % (pic))
            self.callRemote('delText',u"thumbrow%d" %(j+1),u"thumb%sitem" % (pic))
            self.images.pop(pic)

    def clearsnaps( self):
        print "in clearsnaps",self.images
        if self.images:
           self.callRemote('delText',u"hproducts",u"products")
           self.callRemote('addText',u'div',u"hproducts",u"Clearing Snapshots",u"products")
           pics = sorted(self.images.keys())
           nchunk = (len(pics)-1)/self.schunk + 1
           for j in range(nchunk):
               self.clearsnap(j)
           self.callRemote('removeRows',u"tablethumbs")

    def snapshot( self):
        print "in snapshot"
        if hasattr(self.page,"callRemote") and self.showS and self.showP:
           self.callRemote('delText',u"hproducts",u"products")
           name = u"%s" % (self.target.replace(" ",""))
           text = u"Products for %s (Pass %d, %s)" % (name,self.showP,self.tstages[self.showP][self.showS-1])
           self.callRemote('addText',u'div',u"hproducts",text,u"products")
           if self.showP == 1:
              if self.showS == 1: tdir = u"%s/mode%d/image" % (self.target,self.showP)
              elif self.showS == 2: tdir = u"%s/sky%d/image" % (self.target,self.showP)
              elif self.showS == 3: tdir = u"%s/shift%d/image" % (self.target,self.showP)
              elif self.showS == 4: tdir = u"%s/Stack%d_" % (self.target,self.showP)
              elif self.showS == 5: tdir = u"%s/Mosaic%d" % (self.target,self.showP)
           elif self.showP == 2:
              if self.showS == 1: tdir = u"%s/mode%d/image" % (self.target,self.showP)
              elif self.showS == 2: tdir = u"%s/back%d/image" % (self.target,self.showP)
              elif self.showS == 3: tdir = u"%s/mall%d/image" % (self.target,self.showP)
              elif self.showS == 4: tdir = u"%s/smow%d/image" % (self.target,self.showP)
              elif self.showS == 5: tdir = u"%s/mbg%d/image" % (self.target,self.showP)
              elif self.showS == 6: tdir = u"%s/sky%d/image" % (self.target,self.showP)
              elif self.showS == 7: tdir = u"%s/shift%d/image" % (self.target,self.showP)
              elif self.showS == 8: tdir = u"%s/Stack%d_" % (self.target,self.showP)
              elif self.showS == 9: tdir = u"%s/Mosaic%d" % (self.target,self.showP)
           if u"Stack" in tdir: self.schunk = 3
           elif u"shift" in tdir: self.schunk = 2
           else: self.schunk = 2
           self.thumbs("/"+tdir)

    def waitFor(self,d):
        while not d.called:
            print "Waiting on",d,d.called
            time.sleep(1)
        print "Called:",d,d.called

    def dothumb(self,r,j,pic,tdir):
        suff = u"%s" % (pic.split(".")[-1])
        lpic = u"%s" % (pic.replace("."+suff,"link"))
        upic = u"%s" % (tdir+pic.replace("mag08."+suff,"mag01."+suff))
        fpic = u"%s" % (pic.replace("mag08","").replace("."+suff,""))
        finfo = [
                fpic,
                ""
                ]

        if "shift%d" % (self.showP) in tdir:
           frame = glob(u"%s/%s/sky%d/%sc?.fits" % (self.runner.basedir,self.target,self.showP,fpic))
           if frame:
              F = FITS(frame[0])
              xdiff1 = F["xdiff1"]
              ydiff1 = F["ydiff1"]
              mdiff1 = F["mdiff1"]
              xscat1 = F["xscat1"]
              yscat1 = F["yscat1"]
              mscat1 = F["mscat1"]
              lmatch1 = F["lmatch1"]
              F.close()
              finfo = [
                      u"%s %6.2f %6.2f %6.2f" % (fpic,xdiff1,ydiff1,mdiff1),
                      u"%d %6.2f %6.2f %6.2f" % (lmatch1,xscat1,yscat1,mscat1),
                      ]
        elif "mode%d" % (self.showP) in tdir:
           frame = glob(u"%s/%s/mode%d/%s.fits" % (self.runner.basedir,self.target,self.showP,fpic))
           if frame:
              F = FITS(frame[0])
              medval = F["median"]
              scat = F["scat"]
              F.close()
              finfo = [
                      u"%s %8.2f %8.2f" % (fpic,medval,scat),
                      ]
        elif "sky%d" % (self.showP) in tdir:
           frame = glob(u"%s/%s/sky%d/%s.fits" % (self.runner.basedir,self.target,self.showP,fpic))
           if frame:
              F = FITS(frame[0])
              medval = F["median"]
              anoise = F["anoise"]
              F.close()
              enoise = getsqrt(8*medval + 25**2)/8.
              finfo = [
                      u"%s %6.2f %6.2f" % (fpic,anoise,enoise),
                      ]
        else:
           finfo = [
                   fpic,
                   ""
                   ]

        self.callRemote('addElement', u'td', u"border:thin solid black;", u'thumbrow%d' % (r+1), u'', u'thumb%sitem' % (pic))
        self.callRemote('addElement', u'table', u"", u'thumb%sitem' % (pic), u'', u'thumb%stbl' % (pic))
        self.callRemote('addElement', u'tr', u'border: none', u'thumb%stbl' % (pic), u'', u'thumb%srow1' % (pic))
        self.callRemote('addElement', u'td', u"", u'thumb%srow1' % (pic), u'', u'thumb%s' % (pic))
        self.callRemote('addElement', u'tr', u'border: none', u'thumb%stbl' % (pic), u'', u'thumb%srow2' % (pic))
        self.callRemote('addElement', u'td', u"", u'thumb%srow2' % (pic), u'', u'thumb%sinfo' % (pic))
        self.callRemote('addElement', u'table', u"", u'thumb%sinfo' % (pic), u'', u'thumb%sinfotbl' % (pic))

        ninfo = len(finfo)
        for j,ftext in enumerate(finfo):
            if ftext:
               self.callRemote('addElement', u'tr', u'border: none', u'thumb%sinfotbl' % (pic), u'', u'thumb%sinforow%d' % (pic,j+1))
               fwords = ftext.split()
               nwords = len(fwords)
               for i in range(nwords):
                   self.callRemote('addElement', u'td', u"", u'thumb%sinforow%d' % (pic,j+1), u"%s" % (fwords[i]), u'thumb%sinfo%d' % (pic,j*nwords+i))

        self.callRemote('addElement', u'a', u'', u'thumb%s' % (pic), u'', lpic)
        self.callRemote('changeElement', u'href', upic, lpic)
        self.callRemote('changeElement', u'target', u"_blank", lpic)
        self.callRemote('addImage', lpic, tdir+pic, pic)
        self.callRemote('changeElement', u'style', u'width:256', pic)
        return finfo

    def thumbs(self,tdir):
        items1 = glob(self.runner.basedir+"/"+tdir+"*.jpg")
        items2 = glob(self.runner.basedir+"/"+tdir+"*.png")
        bdir = "/".join(tdir.split("/")[:-1]) + "/"
        items = sorted(items1 + items2)
        grab = lambda item: item[-9:] in ["mag08.jpg","mag08.png"] or item.split(".")[-1] in ["png"]
        pics = [u"%s" % (os.path.basename(item)) for item in items if grab(item)]
        nchunk = (len(pics)-1)/self.schunk + 1
        for j in range(nchunk):
            self.callRemote('addElement', u'tr', u'border: none', u'tablethumbsbody', u'', u'thumbrow%d' % (j+1))
            for p,pic in enumerate(pics[j*self.schunk:(j+1)*self.schunk]):
                finfo = self.dothumb(j,p+j*self.schunk,pic,bdir)
                self.images[pic] = finfo
                print pic

    def addThumbs(self):
             
        if hasattr(self,"clock") and self.clock.running: self.clock.stop()
        self.fireUpProducts()

    def table( self):
        name = u"%s" % (self.target.replace(" ",""))
        ptitle = lambda p,s: u"(%d) %s" % (p,self.tstages[p][s-1])
        p = T.div(name=u"productTable")[
           T.table(style="border:thin solid black;")[
           T.thead[T.tr[
              T.th(style="width:180;border:thin solid black;")["Stage"],
              T.th(style="width:720;border:thin solid black;",name=u"hproducts")[T.div(name=u"products")["Select a Stage"]],
              ]],
           T.tbody[T.tr[
              T.td(style="border:thin solid black;",name=u"thumbs")[
                   (T.div(name=u"sel%d"%(100*p+s),style="font-weight:normal;color:orange",\
                     onclick='Nevow.Athena.Widget.get(this).selected(\'%d\')' % (100*p+s))[ptitle(p,s)] for p in [1] for s in self.nstages[p]),
                   T.br,
                   (T.div(name=u"sel%d"%(100*p+s),style="font-weight:normal;color:orange",\
                     onclick='Nevow.Athena.Widget.get(this).selected(\'%d\')' % (100*p+s))[ptitle(p,s)] for p in [2] for s in self.nstages[p])
                   ],
              T.td(style="border:thin solid black;")[
                  T.div(name=u"showthumbs")[T.div(name=u"showstage")[
                  T.div(name=u"stagethumbs",style=u'width:720;height:580;overflow:auto')[
                        T.table(name=u"tablethumbs")[
                          T.thead(name=u"tablethumbshead"),
                          T.tbody(name=u"tablethumbsbody")
                        ]
                        ]
                  ]]
              ]
              ]
              ]
        ]
        ]
        return p

    def makeDocFactory( self):
        return loaders.stan(T.div(render=T.directive('liveElement'))[
             T.h2(name="selheader")[T.div(name="seltarget")[self.target]],
               self.table()
             ])

def StartSite(runner,port):
    site = appserver.NevowSite(MainPage(runner=runner))
    application = service.Application("NEWFIRM TV")
    reactor.listenTCP(port,site)



